<?php

namespace Drupal\grant;

use Drupal\Core\Cache\Cache;
use Drupal\multiple_email\EmailInterface;
use Drupal\user\Entity\User;

/**
 * Provide main functions for grant module.
 */
class GrantMain {

  /**
   * Retrieve grant config of an entity type.
   */
  public function getGrantConfig($entity_type) {
    $grant_configs = $this->getGrantConfigs();
    return $grant_configs[$entity_type];
  }

  /**
   * Retrieve grant config of all entity types.
   */
  public function getGrantConfigs() {
    // @todo: add caching here.
    $config_storage = \Drupal::entityTypeManager()->getStorage('grant_config');
    $config_entities = $config_storage->loadByProperties(['status' => TRUE]);
    $grant_configs = [];
    foreach ($config_entities as $entity_type => $config_entity) {
      $grant_configs[$entity_type] = [
        'assignroles' => $config_entity->getAssignRoles(),
        'parentfields' => $config_entity->getParentFields()
      ];
    }
    return $grant_configs;
  }

  /**
   * Check if user has permission via assigned grant roles.
   */
  public function userAssignedGrantHasPermission($user_uuid = 'current', $permission_raw = '', $e_type = '', $e_uuid = '') {
    $uag_rids = $this->getUserAssignedGrantRoles($user_uuid, $e_type, $e_uuid);
    $rolestorage = \Drupal::entityTypeManager()->getStorage('user_role');

    // Logic from "/core/modules/user/src/Access/PermissionAccessCheck.php":
    $conjunction = 'AND';
    $split = explode(',', $permission_raw);
    if (count($split) > 1) {
      $conjunction = 'AND';
    }
    else {
      $split = explode('+', $permission_raw);
      $conjunction = 'OR';
    }
    // Logic from "/core/lib/Drupal/Core/Access/AccessResult.php":
    $access = FALSE;
    if ($conjunction == 'AND' && !empty($permissions)) {
      $access = TRUE;
      foreach ($split as $permission) {
        if (!$permission_access = $rolestorage->isPermissionInRoles($permission, $uag_rids)) {
          $access = FALSE;
          break;
        }
      }
    }
    else {
      foreach ($split as $permission) {
        if ($permission_access = $rolestorage->isPermissionInRoles($permission, $uag_rids)) {
          $access = TRUE;
          break;
        }
      }
    }
    return $access;
  }

  /**
   * Retrieve assigned grant roles of a user.
   */
  public function getUserAssignedGrantRoles($user_uuid = 'current', $e_type = '', $e_uuid = '') {
    if ($user_uuid == 'current') {
      $user_id = \Drupal::currentUser()->id();
      $user = User::load($user_id);
      $user_uuid = $user->uuid();
    }
    else {
      $user = \Drupal::service('entity.repository')->loadEntityByUuid('user', $user_uuid);
    }

    // At first load core (global) assigned roles of the user.
    $uag_rids = $user->getRoles();
    if ($e_type == "" && $e_uuid == "") {
      // Return just global assigned role ids.
      return $uag_rids;
    }

    $uag = $this->getUserAssignedGrants($user_uuid);
    $data = $this->getUserAssignedGrantEntitiesRoles(
      [
        'grant_config' => $this->getGrantConfigs(),
        'uag_rids' => $uag_rids,
        'uag' => $uag,
        'e_type' => $e_type,
        'e_uuid' => $e_uuid,
        'e_processed' => [],
        'counter' => 0,
      ]);
    return $data['uag_rids'];
  }

  /**
   * Retrieve user assigned grants roles of entities incl. parent entities.
   */
  public function getUserAssignedGrantEntitiesRoles($data) {
    if (isset($data['e_processed'][$e_type])
      && isset($data['e_processed'][$e_type][$e_uuid])) {
      return $data;
    }
    $uag = $data['uag'];
    $e_type = $data['e_type'];
    $e_uuid = $data['e_uuid'];
    $uag_rids_entity = [];
    if (isset($uag[$e_type]) && isset($uag[$e_type][$e_uuid])) {
      $uag_rids_entity = array_keys($uag[$e_type][$e_uuid]);
      $data['e_processed'][$e_type][$e_uuid] = $uag_rids_entity;
    }
    $data['uag_rids'] = array_merge($uag_rids_entity, $data['uag_rids']);

    // Check if current entity type has referenced parent fields defined.
    if (isset($data['grant_config'][$e_type])
      && isset($data['grant_config'][$e_type]['parentfields'])) {
      if ($data['grant_config'][$e_type]['parentfields'] != []) {
        $entity = \Drupal::service('entity.repository')
          ->loadEntityByUuid($e_type, $e_uuid);
        if ($entity) {
          $fields = $entity->getFields();
          // Only loop through referenced parent fields.
          foreach ($data['grant_config'][$e_type]['parentfields'] as $parent) {
            if (isset($fields[$parent])) {
              $field = $fields[$parent];
              $data['e_type'] = $field->getSetting('target_type');
              $values = $field->getValue();
              foreach ($values as $value) {
                if (isset($value['target_uuid'])) {
                  $data['e_uuid'] = $value['target_uuid'];
                }
                else {
                  $data['e_uuid'] = "0";
                  if (isset($value['target_id'])) {
                    $target = \Drupal::entityTypeManager()
                      ->getStorage($data['e_type'])
                      ->load($value['target_id']);
                    if ($target) {
                      $data['e_uuid'] = $target->uuid();
                    }
                  }
                }
                // Self calling this function.
                $data = $this->getUserAssignedGrantEntitiesRoles($data);
              }
            }
          }
        }
      }
    }
    $data['counter']++;
    // @todo: Exit strategy and error message on counter limit.
    return $data;
  }

  /**
   * Retrieve user assigned grants.
   */
  public function getUserAssignedGrants($uuid = "current") {
    if ($uuid == "current") {
      $uuid = User::load(\Drupal::currentUser()->id())->uuid();
    }
    $cache_id = 'grant_uag:' . $uuid;
    $cached = \Drupal::cache()->get($cache_id);
    if ($cached) {
      $grants = $cached->data;
    }
    else {
      $grants = [];
      $grant_storage = \Drupal::entityTypeManager()->getStorage('grant');

      $grant_query = $grant_storage->getQuery()
        ->condition('assignee', $uuid)
        ->condition('status', 1);
      $grant_entities = $grant_storage->loadMultiple($grant_query->execute());
      foreach ($grant_entities as $grant_id => $grant) {
        $role = $grant->get('role')->value;
        $entity_type = $grant->get('entity_type')->value;
        $entity_uuid = $grant->get('entity_uuid')->value;
        $grants[$entity_type][$entity_uuid][$role] = $grant->get('uuid')->value;
      }
      $cache_tags = ['grant_uag', 'grant:' . $grant_id];
      \Drupal::cache()->set($cache_id, $grants, Cache::PERMANENT, $cache_tags);
    }
    return $grants;
  }

  /**
   * Retrieve user mail addresses.
   */
  public function getUserEmails($uid = 0) {
    if ($uid == 0) {
      $uid = \Drupal::currentUser()->id();
    }
    $emails = [];
    if (\Drupal::moduleHandler()->moduleExists('multiple_email')) {
      /** @var \Drupal\multiple_email\EmailStorageInterface $email_storage */
      $email_storage = \Drupal::entityTypeManager()->getStorage('multiple_email');

      $multiple_email_query = $email_storage->getQuery()
        ->condition('uid', $uid)
        ->condition('status', EmailInterface::CONFIRMED);
      $email_entities = $email_storage->loadMultiple($multiple_email_query->execute());

      foreach ($email_entities as $email_entity) {
        $emails[] = $email_entity->getEmail();
      }
    }
    else {
      if ($uid == \Drupal::currentUser()->id()) {
        $user = \Drupal::currentUser();
      }
      else {
        $user = User::load($uid);
      }
      $emails[] = $user->getEmail();
    }
    return $emails;
  }

  /**
   * Helper function.
   *
   * Transform a string from a multiline textfield to an array of line strings.
   */
  public function textfieldToArray($multilinestring) {
    $normalize_lines = trim(str_replace(["\r\n", "\n\r", "\r"], "\n", trim($multilinestring)));
    if ($normalize_lines == "") {
      return [];
    }
    return explode("\n", $normalize_lines);
  }

}
